#' @title Trace of a matrix
#' @description Calculate the trace of a matrix.
#'
#' @param A A quadratic matrix.
#' @return The trace of \code{A}.
#' @examples
#' TraceMatrix(cbind(c(1,2),c(3,4)))
#' @export
TraceMatrix <- function(A) sum(diag(A)); # to calculate the trace of a matrix

#' @title Norm of a matrix
#' @description Calculate the Frobenius norm of a matrix.
#'
#' @param A A matrix.
#' @return The norm of \code{A}.
#' @examples
#' NormMatrix(cbind(c(1,2),c(3,4)))
#' @export
NormMatrix <- function(A) sqrt(TraceMatrix(t(A)%*%A)); # to calculate the norm of a matrix

#' @title Quadratic norm of complex number
#' @description Calculate the quadratic norm of a complex number
#'
#' @param b A complex number.
#' @return The quadratic norm of \code{b}.
#' @examples
#' NormNumberInC(3+4i)
#' @export
NormNumberInC <- function(b) Re((Conj(b)*b)); # quadratic norm for complex numbers

#' @title k-dimensional Helmert-Submatrix
#' @description Get k-dimensional Helmert-Submatrix.
#'
#' @param k Dimension of Helmert-Submatrix.
#' @return The \code{k}-dimensional Helmert-Submatrix.
#' @examples
#' HelmertSubMatrix(5)
#' @export
HelmertSubMatrix <- function(k){
  z  <- matrix(0, ncol=k-1, nrow=k, byrow=TRUE);
  for (i in 1:k-1) {# each column
    z[,i] <- c(rep(1/sqrt(i*(i+1)), i), -i/sqrt(i*(i+1)), rep(0, k-i-1));
  }
  z;
}

#' @title Center two-dimensional patterns
#' @description Center two-dimensional fingerprint patterns or list thereof.
#'
#' @param list1 A two-dimensional fingerprint pattern or a list consisting of two-dimensional fingerprint patterns.
#' @return A two-dimensional centered fingerprint pattern or a list consisting of two-dimensional centered fingerprint patterns.
#' @examples
#' CenterFingerprints(FirstFingers)
#' @export
CenterFingerprints <- function(list1){
  if (typeof(list1) == "double") {
    return(list1%*%HelmertSubMatrix(dim(list1)[2])%*%t(HelmertSubMatrix(dim(list1)[2])))
  }
  else if (typeof(list1) == "list"){
    returnList <- list()
    for (i in 1:length(list1)){
      returnList[[i]] <- list1[[i]]%*%HelmertSubMatrix(dim(list1[[i]])[2])%*%t(HelmertSubMatrix(dim(list1[[i]])[2]))
    }
    return(returnList)
  }

  else{
    print("Please, use only 2xn-dimensional matrices of type double or lists thereof.")
    return(FALSE)
  }
}

#' @title Estimate anisotropic growth parameters
#' @description Calculate the rotational angle, the rate of isotropic growth, the rate of anisotropic growth and the angle of anisotropic growth for two fingerprints.
#'
#' @keywords internal
#' @param OtherFinger Matrix of centered minutiae coordinates.
#' @param FirstFinger Matrix of centered minutiae coordinates to be compared with.
#' @param epsilon Accuracy parameter.
#' @param betaStart First estimation for beta.
#' @param lambdaStart First estimation for lambda.
#' @param gammaStart First estimation for gamma.
#' @param tauStart First estimation for tau.
#' @param iterationsMax Maximal number of iterations.
#' @return Named list with the components
#' \itemize{
#'  \item \code{lambda} Estimation for isotropic growth rate with accuracy \code{epsilon}.
#'  \item \code{tau} Estimation for rate of anisotropic growth with accuracy \code{epsilon}.
#'  \item \code{beta} Estimation for rotation angle with accuracy \code{epsilon}.
#'  \item \code{gamma} Estimation for direction of anisotropic growth with accuracy \code{epsilon}.
#'  \item \code{no_iterations} Number of iterations of the algorithm.
#' }
#' @examples
#' Minimization(OtherFingersCentered[[1]], FirstFingersCentered[[1]])
#' @export
Minimization <- function(FirstFinger, OtherFinger, epsilon=10^(-10), betaStart = pi, lambdaStart = 1.5, gammaStart = 0.5*pi, tauStart = 0.5, iterationsMax = 1000){
  Z_2 <- OtherFinger[1,]+1i*OtherFinger[2,]
  Z_1 <- FirstFinger[1,]+1i*FirstFinger[2,]
  # Setze Anfangswerte
  no_iterations <- 0;
  tau <- tauStart;
  tau_alt <- tau;
  beta <- betaStart;
  beta_alt <- beta;
  gamma <- gammaStart;
  gamma_alt <- gamma;
  lambda <- lambdaStart;
  lambda_alt <- lambda;
  abbruch <- FALSE;

  # Iterativer Aufruf mit Abbruchbedingung
  while(abbruch == FALSE){
    no_iterations <- no_iterations+1;

    # Setze oben auf null
    oben <- 0;

    # Berechnung fuer beta
    for (k in 1:length(Z_1)) {
      A <- (Z_1[k] + tau*Re(Conj(Z_1[k])*exp(1i*gamma))*exp(1i*gamma));
      oben <- oben + Z_2[k]*Conj(A);
    }
    unten <- Conj(oben);

    if (is.na(oben/unten)){ # Kontrolle, dass nicht durch null geteilt wird
      beta <- 0;
    } else {
      beta1 <- Arg(oben/unten)/2;
      beta2 <- Arg(oben/unten)/2 + pi;
      if (abs(beta - beta1) < abs(beta - beta2)){ # Fallunterscheidung zur Auswahl von beta
        beta <- beta1;
      } else {
        beta <- beta2;
      }
    }

    # Setze oben und unten auf null
    oben <- 0;
    unten <- 0;

    # Berechnung fuer lambda
    for (k in 1:length(Z_1)) {
      J <- Z_2[k];
      I <- exp(1i*beta)*(Z_1[k]*(1 + tau/2) + (tau/2)*exp(2i*gamma)*Conj(Z_1[k]));
      oben <- oben + Re(Conj(J)*I);
      unten <- unten + NormNumberInC(I);
    }
    if (is.na(oben/unten)) { # Kontrolle, dass nicht durch null geteilt wird
      lambda <- 0;
    } else {
      lambda <- oben/unten;
    }

    # Setze oben und unten auf null
    oben <- 0;
    unten <- 0;

    # Berechnung fuer gamma
    for (k in 1:length(Z_1)) {
      C <- Z_2[k]-lambda*exp(1i*beta)*Z_1[k]*(1 + tau/2);
      oben <- oben + C*exp(-1i*beta)*Z_1[k];
    }
    unten <- Conj(oben);

    if (is.na(oben/unten)){ # Kontrolle, dass nicht durch null geteilt wird
      gamma <- 0;
    } else {
      gamma1 <- Arg(oben/unten)/4;
      gamma2 <- Arg(oben/unten)/4 + pi/2;
      if (abs(gamma-gamma1) < abs(gamma-gamma2)){ # Fallunterscheidung zur Auswahl von gamma
        gamma <- gamma1%%pi;
      } else {
        gamma <- gamma2%%pi;
      }
    }

    # Setze oben und unten auf null
    oben <- 0;
    unten <- 0;

    # Berechnung fuer tau
    for (k in 1:length(Z_1)) {
      B <- Z_2[k]-lambda*exp(1i*beta)*Z_1[k];
      oben <- oben + Re(Conj(Z_1[k])*exp(1i*gamma))*Re(Conj(B)*exp(1i*(gamma+beta)));
      unten <- unten + Re(Conj(Z_1[k])*exp(1i*gamma))^2;
    }
    if (is.na(oben/unten)){ # Kontrolle, dass nicht durch null geteilt wird
      tau <- 0;
    } else {
      tau <- oben/(lambda*unten);
    }

    # Abbruch-Bedingung
    tau <- Re(tau);
    lambda <- Re(lambda);
    gamma <- Re(gamma);
    beta <- Re(beta) %% (2*pi);

    if (abs(exp(2*1i*gamma_alt) - exp(2*1i*gamma))
        + abs(tau_alt - tau)
        + abs(exp(1i*beta_alt) - exp(1i*beta))
        + abs(lambda_alt - lambda) < epsilon) {
      abbruch <- TRUE;
    }

    # Aktualisierung der Vergleichswerte
    gamma_alt <- gamma;
    tau_alt <- tau;
    beta_alt <- beta;
    lambda_alt <- lambda;

    if (no_iterations >= iterationsMax){
      print("Maximal number of iterations reached.")
      abbruch <- TRUE;
    }
  }

  # Umrechnungen

  lambda_alt1 <- lambda;
  tau_alt1 <- tau;
  beta_alt1 <- beta;
  gamma_alt1 <- gamma;

  if (Re(tau) < 0){
    lambda <- (1+tau_alt1)*lambda_alt1;
    tau <- (-lambda + lambda_alt1)/lambda;
    gamma <- (gamma - pi/2)%%pi;
    wneu <- rbind(cos(gamma), sin(gamma));
  }

  if (Re(lambda) < 0){
    lambda <- -lambda;
    beta <- (beta + pi) %% (2*pi);
    Rotation <- cbind(c(cos(beta), sin(beta)), c(-sin(beta), cos(beta)));
  }
  return (list("lambda" = lambda, "tau" = tau, "beta" = beta, "gamma" = gamma,
               "no_iterations" = no_iterations))
}

#' @title Estimate anisotropic growth parameters
#' @description Calculate the rotational angle (beta), the rate of isotropic growth (lambda), the rate of anisotropic growth (tau) and the angle of anisotropic growth (gamma) for two fingerprints or lists thereof.
#'
#' @param OtherFingers Matrix of centered minutiae coordinates or lists thereof.
#' @param FirstFingers Matrix of centered minutiae coordinates to be compared with or lists thereof.
#' @param epsilon Accuracy parameter.
#' @param betaStart First estimation for beta.
#' @param lambdaStart First estimation for lambda.
#' @param gammaStart First estimation for gamma.
#' @param tauStart First estimation for tau.
#' @param iterationsMax Maximal number of iterations.
#' @return Named list with the components
#' \itemize{
#'  \item \code{lambda} Estimation for isotropic growth rate with accuracy \code{epsilon}.
#'  \item \code{tau} Estimation for rate of anisotropic growth with accuracy \code{epsilon}.
#'  \item \code{beta} Estimation for rotation angle with accuracy \code{epsilon}.
#'  \item \code{gamma} Estimation for direction of anisotropic growth with accuracy \code{epsilon}.
#'  \item \code{no_iterations} Number of iterations of the algorithm.
#' }
#' @examples
#' AnisotropicGrowthEstimation(FirstFingersCentered, OtherFingersCentered)
#' boxplot(Arg(exp(1i*AnisotropicGrowthEstimation(FirstFingersCentered, OtherFingersCentered)$beta)), main="betas")
#' @export
AnisotropicGrowthEstimation <- function(FirstFingers, OtherFingers, epsilon=10^(-10), betaStart = pi, lambdaStart = 1.5, gammaStart = 0.5*pi, tauStart = 0.5, iterationsMax = 1000) {
  if ((typeof(FirstFingers) == "double") && typeof(OtherFingers) == "double"){
    return (Minimization(FirstFingers, OtherFingers, epsilon=epsilon, tauStart = tauStart, gammaStart = gammaStart, lambdaStart = lambdaStart, iterationsMax = iterationsMax))  # changed the order Mini
  }
  else if ((typeof(FirstFingers) == "list") && typeof(OtherFingers) == "list"){
    lambda <- c();
    tau <- c();
    beta <- c();
    gamma <- c();
    no_iterations <- c();
    for (i in 1:length(FirstFingers)){
      MinResult <- Minimization(FirstFingers[[i]], OtherFingers[[i]], epsilon=epsilon, betaStart = betaStart, tauStart = tauStart, gammaStart = gammaStart, lambdaStart = lambdaStart, iterationsMax = iterationsMax)
      lambda <- c(lambda, MinResult$lambda)
      tau <- c(tau, MinResult$tau)
      beta <- c(beta, MinResult$beta)
      gamma <- c(gamma, MinResult$gamma)
      no_iterations <- c(no_iterations, MinResult$no_iterations)
    }
    return (list("lambda" = lambda, "tau" = tau, "beta" = beta, "gamma" = gamma,
                 "no_iterations" = no_iterations))
  }
}

#' @title Simulate anisotropic growth
#' @description Simulate anisotropic growth for given corresponding point patterns.
#'
#' @param Fingers Matrix of minutiae coordinates or lists thereof.
#' @param gamma Angle of anisotropic growth.
#' @param taus Rate of anisotropic growth (constant or vector).
#' @param lambdas Rate of isotropic growth (constant or vector).
#' @return List of centered fingerprints grown isotropically by lambda and with anisotropic growth determined by gamma and tau.
#' @examples
#' lambda = 1
#' tau = 0
#' estimatedResults <- AnisotropicGrowthEstimation(FirstFingers = FirstFingersCentered, OtherFingers = SimulateAnisotropicGrowth(OtherFingersOrientedCentered, taus = tau,lambdas=lambda))
#' boxplot(estimatedResults$lambda)
#' boxplot(estimatedResults$tau)
#' @export
SimulateAnisotropicGrowth<-function(Fingers,gamma = 0,taus = 0,lambdas=1){
  if (typeof(Fingers) == "double"){
    nr_prints = 1
  }
  else if (typeof(Fingers) == "list"){
    nr_prints = length(Fingers)
  }
  #Initialisierung neue Daten, Input sind Listen mit jeweils 56 Punktmusters,
  # offenbar ist Z1neu unwichtig, man braucht nur Z2neu, das muss offenbar orientiert auf null sein und wachsen
  #Z1gewachsen<-list()
  Z2gewachsen<-list()
  # Wachstumsparameter
  # wir erlauben taus = Zahl oder = Vektor mit verschiedenem Wachstum f?r mehr Realismus

  wneu<-c(cos(gamma),sin(gamma))

  if (length(taus) == 1) taus <- rep(taus, nr_prints)    # downward compatibility
  if (length(lambdas) == 1) lambdas <- rep(lambdas, nr_prints)    # downward compatibility

  # beide Listen zentrieren
  #Z_1neuz<-list()
  Z_2neuz<-list()
  for(S in 1:nr_prints){
    Z_2neuz[[S]]<-Fingers[[S]]%*%HelmertSubMatrix(dim(Fingers[[S]])[2])%*%t(HelmertSubMatrix(dim(Fingers[[S]])[2]))
    #Z_2neuz[[S]]<-OtherFingers[[S]]%*%HelmertSubMatrix(dim(OtherFingers[[S]])[2])%*%t(HelmertSubMatrix(dim(OtherFingers[[S]])[2]))
  }

  for(i in 1:nr_prints){
    Z2gewachsen<-append(Z2gewachsen,list(lambdas[i]*(Z_2neuz[[i]] + taus[i]*wneu%*%t(wneu)%*%Z_2neuz[[i]])))
    #   Z2gewachsen<-append(Z2gewachsen,list((Z_2neuz[[i]] + taus*wneu%*%t(wneu)%*%Z_2neuz[[i]])))
  }
  return(Z2gewachsen)
}

#' @title Rose Diagram for given angles
#' @description Rose Diagram for given angles.
#'
#' @param gamma Vector of angles in [0, pi) or [-pi/2, pi/2).
#' @param nr_bins Number of bins for histogram.
#' @param main_chosen Text for main.
#' @examples
#' RoseDiagram(AnisotropicGrowthEstimation(FirstFingersCentered, OtherFingersCentered)$gamma - pi/2)
#' @export
RoseDiagram <- function(gamma, print.sub = FALSE, nr_bins = 20, cex = 2, main=expression(paste("Doubled ",gamma))){
  R_quer <- abs(sum(exp(2*1i*gamma)))/length(gamma)
  extrinsic_mean <- Arg(sum(exp(2*1i*gamma)))
  if (print.sub==TRUE) 
      sub <- paste("Extrinsic mean = ",format(extrinsic_mean, digits=2),"resultant length = ",format(R_quer, digits=2))
  else
      sub <- NULL
  rose.diag(2*gamma,bins=nr_bins, main=main, sub=sub,cex=cex);
  arrows(0, 0, R_quer*cos(extrinsic_mean),R_quer*sin(extrinsic_mean),col="red", lwd=2, length=0.1)
       textxy(-0.2,-0.3, labs=substitute(paste(hat(R), " =  ", var1),list(var1=format(R_quer,digits=2))), m = c(0, 0),cex=2,offset=0,col="red")
}

#' @title Circular point diagram
#' @description Circular point diagram.
#'
#' @param gamma Vector of angles in [0, pi) or [-pi/2, pi/2).
#' @examples
#' PointDiagram(AnisotropicGrowthEstimation(FirstFingersCentered, OtherFingersCentered)$gamma - pi/2)
#' @export
PointDiagram <- function(gamma, main="2 gammas"){
  R_quer <- abs(sum(exp(2*1i*gamma)))/length(gamma)
  extrinsic_mean <- Arg(sum(exp(2*1i*gamma)))
  gamma_2pi <- 2*gamma
  plot(cos(gamma_2pi[1]),sin(gamma_2pi[1]), sub=paste("Residual length = ",format(R_quer,digits=2)),main=main,xlim=c(-1.5,1.5),ylim=c(-1.5,1.5),axes=FALSE,xlab="",ylab="",asp=1,pch=19)
  for(i in 2:length(gamma_2pi)){
    points(cos(gamma_2pi[i]),sin(gamma_2pi[i]),pch=19)
  }
  points(cos(0),sin(0),pch=3)
  points(cos(pi/2),sin(pi/2),pch=3)
  points(cos(pi),sin(pi),pch=3)
  points(cos(3*pi/2),sin(3*pi/2),pch=3)
  points(0,0,pch=3)
  textxy(1.15,0, labs=0, m = c(0, 0),cex=1,offset=0)
  textxy(cos(pi)-0.15,sin(pi), labs=expression(pi), m = c(0, 0),cex=1,offset=0)
  draw.circle(x=0,y=0,radius=1,lwd=1.2)
  arrows(0, 0, R_quer*cos(extrinsic_mean),R_quer*sin(extrinsic_mean),col="red", lwd=2, length=0.1)
}

#' @title Simulation of fingerprint patterns
#' @description Simulation of fingerprint patterns with Poisson distributed number of minutiae. The coordinates are i.i.d. uniformly distributed within the assigned interval for x and y.
#'
#' @param sigma Parameter for normal distribution of error.
#' @param poissonExpectedValue Parameter for Poisson distribution.
#' @param nr_fingers Number of fingers.
#' @param nr_printsPerFinger Number of fingerprints per finger.
#' @param seed Seed for random distributions.
#' @param use_seeds Boolean value, \code{TRUE} if seed is used.
#' @param xlimits Vector containing the minimum and the maximum x-value of the minutiae.
#' @param ylimits Vector containing the minimum and the maximum y-value of the minutiae.
#' @return Named list with the components
#' \itemize{
#'  \item \code{FirstFingers} List of minutiae patterns for simulated first fingers.
#'  \item \code{OtherFingers} List of minutiae patterns for simulated other fingers.
#' }
#' @examples SimulatePoissonFingers()
#' @export
# SimulatePoissonFingers <- function(sigma = 10, poissonExpectedValue = 40, nr_fingers = 8, nr_printsPerFinger = 7, seed = 1, use_seed = FALSE, xlimits = c(-400,400), ylimits = c(-50,50)){
SimulatePoissonFingers <- function(sigma = 7, poissonExpectedValue = 22, nr_fingers = 8, nr_printsPerFinger = 7, seed = 1, use_seed = FALSE, xlimits = c(-95,95), ylimits = c(-165,165)){
  if (use_seed == TRUE){
    set.seed(seed)
  }
  Z1neu<-list()
  Z2neu <- list()
  bottom.factor <- 5; top.factor <- 5
  for (fingerprint_nr in 1:nr_fingers){
    n<-rpois(1, poissonExpectedValue) #Anzahl der Elemente der Punktwolke
    #Erzeuge die Koordinaten der Punktwolke
    x1<-runif(n, xlimits[1],xlimits[2])
    y1<-runif(n, ylimits[1],ylimits[2])
    for(i in 1:nr_printsPerFinger){
      x2<-x1+rtruncnorm(n, a=-sigma*bottom.factor, b=sigma*top.factor, mean = 0, sd = sigma)
      y2<-y1+rtruncnorm(n, a=-sigma*bottom.factor, b=sigma*top.factor, mean = 0, sd = sigma)
      Z1neu<-append(list(rbind(x1,y1)),Z1neu)
      Z2neu<-append(list(rbind(x2,y2)),Z2neu)
    }
  }
  return(list("FirstFingers" = CenterFingerprints(Z1neu), "OtherFingers" = CenterFingerprints(Z2neu)))
}

#' @title Test for gamma
#' @description Rayleight-test for anisotropy of gamma. Null-hypothesis: There is no anisotropic growth effect.
#'
#' @param alpha Confidence level.
#' @param gamma Vector of angles in [0, pi).
#' @return Named list with the components
#' \itemize{
#' \item \code{reject} \code{TRUE}, if null-hypothesis is rejected. Otherwise, \code{FALSE}.
#' \item \code{test.statistic} The Rayleigh test statistic.
#' \item \code{critical.value} The 1-alpha quantile of the chisquare distribution with two degrees of freedom.
#' }
#' @examples
#' TestForGamma(AnisotropicGrowthEstimation(FirstFingersCentered, OtherFingersCentered)$gamma - pi/2)
#' tau_values = c()
#' for (gamma_value in seq(0,pi,length=20)){
#' gamma_value = gamma_value + pi/2 #since the fingers are oriented towards north
#' tau = 0
#' while (TestForGamma(AnisotropicGrowthEstimation(FirstFingers = FirstFingersOrientedCentered, OtherFingers = SimulateAnisotropicGrowth(OtherFingersOrientedCentered,gamma = gamma_value,taus = tau,lambdas=1))$gamma - pi/2, alpha = 0.05)$reject == FALSE){
#' tau = tau + 0.001
#' }
#' tau_values = c(tau_values, tau
#' }
#' plot(tau_values, seq(0,pi,length = 20))
#' @export
TestForGamma <- function(gamma, alpha = 0.05){
  R_quer <- abs(sum(exp(2*1i*gamma)))/length(gamma)
  test.statistic <- 2*length(gamma)*R_quer^2
  p.value <- pchisq(test.statistic,2)
  critical.value <- qchisq(1-alpha,2) 
  if( test.statistic > critical.value ){
    print(paste("We reject the hypothesis of isotropic growth with confidence", 1-alpha, "because", test.statistic, " > " ,critical.value))
    reject <- TRUE
  } else{
    print(paste("We do not reject the hypothesis of isotropic growth with confidence", 1-alpha, "because", test.statistic, " <= " , critical.value))
    reject <- FALSE
  }
  return(list("reject"=reject, "test.statistic"=test.statistic, "critical.value"=critical.value, "p.value" = p.value))
}

#' @title Test for tau
#' @description Kolmogorov-Smirnov goodness of fit test for equality of distributions. Null-hypothesis: There is no anisotropic growth effect.
#'
#' @param alpha Confidence level.
#' @param tau Vector wih rates of anisotropic growth.
#' @param sim_taus Vector with rates of simulated isotropic growth.
#' @return Named list with the components
#' \itemize{
#' \item \code{reject} \code{TRUE}, if null-hypothesis is rejected. Otherwise, \code{FALSE}.
#' \item \code{p.value} The p.value of the Kolmogorov-Smirnov test statistic.
#' \item \code{alpha} Confidence level.
#' }
#' @examples
#' generatedFingers = SimulatePoissonFingers()
#' sim_taus = AnisotropicGrowthEstimation(generatedFingers$FirstFingers, generatedFingers$OtherFingers)$tau
#' TestForTau(tau = AnisotropicGrowthEstimation(FirstFingersCentered, OtherFingersCentered)$tau, reference_taus = sim_taus)
#' tau_values = c()
#' generatedFingers = SimulatePoissonFingers(use_seed = TRUE)
#' sim_taus = AnisotropicGrowthEstimation(generatedFingers$FirstFingers, generatedFingers$OtherFingers)$tau
#' for (gamma_value in seq(0, pi, length=20)){
#' gamma_value = gamma_value - pi/2
#' tau = 0
#' TestForTau(tau = AnisotropicGrowthEstimation(FirstFingers = FirstFingersCentered, OtherFingers = SimulateAnisotropicGrowth(OtherFingersOrientedCentered,gamma = gamma_value,taus = tau,lambdas=1))$tau, reference_taus = sim_taus)
#' while (TestForTau(tau = AnisotropicGrowthEstimation(FirstFingers = FirstFingersCentered, OtherFingers = SimulateAnisotropicGrowth(OtherFingersOrientedCentered,gamma = gamma_value,taus = tau,lambdas=1))$tau, reference_taus = sim_taus)$reject == TRUE){
#' tau = tau + 0.001
#' }
#' tau_values = c(tau_values, tau)
#' }
#' plot(tau_values, seq(0, pi))
#' @export
TestForTau <- function(tau, reference_taus, alpha = 0.05){
  p.value <- ks.test(tau,reference_taus)$p
  if(p.value < alpha){
    print(paste("We reject the hypothesis of isotropic growth with confidence", 1-alpha, "because ", p.value, " < " , alpha))
    reject <- TRUE
  }else{
    print(paste("We do not reject the hypothesis of isotropic growth with confidence", 1-alpha, "because ", p.value, " >= " , alpha))
    reject <- FALSE
  }
  return(list("reject"=reject, "p.value"=p.value, "alpha"=alpha))
}

#' @title Test for distal growth
#' @description Parametric and non-parametric test for distal growth.
#'
#' @param alpha Confidence level.
#' @param gamma Vector of angles in [0, pi).
#' @param epsilon Doubled alignment precision for fingerprints (for parametric test only).
#' @param parametric Boolean value, \code{TRUE} if parametric test is used.
#' @param nr_fingers Number of fingers (for non-parametric test only).
#' @param nr_printsPerFinger Number of fingerprints per finger (for non-parametric test only).
#' @return Named list with components
#' \itemize{
#' \item \code{mu_hat} Extrinsic mean (for parametric test)
#' \item \code{delta_res} (for parametric test)
#' \item \code{T_compare} (for non-parametric test)
#' \item \code{qnorm.alpha} \code{qnorm(1-alpha)}. (for non-parametric test)
#' }
#' @export
TestForDistalGrowth <- function(gamma, alpha = 0.05, epsilon = 0.15, parametric = TRUE, nr_fingers = 8, nr_printsPerFinger = 7) {
  if (parametric == TRUE){
    K<-length(gamma)

    R_quer<-(1/K)*abs(sum(exp(2*1i*gamma)))

    kappa_hut<-c()
    if(R_quer<0.53){
      kappa_hut<-c(kappa_hut,2*R_quer+R_quer^3+(5/6)*R_quer^5)
    }
    if(R_quer>=0.53 && R_quer<0.85){
      kappa_hut<-c(kappa_hut,-0.4+1.39*R_quer+0.43*(1/(1-R_quer)))
    }
    if(R_quer>=0.85){
      kappa_hut<-c(kappa_hut,1/(2*(1-R_quer)))
    }

    f_1<-function(gamma){
      exp(kappa_hut*R_quer*cos(gamma))/(2*pi)
    }
    I_j<-integrate(f_1,0,2*pi)$value

    f_list<-list()
    f<-function(theta){
      (1/I_j)*exp(kappa_hut*R_quer*cos(theta-epsilon))*(1/(2*pi))
    }
    f_list<-append(f_list,list(f))
    
    all_deltas<-c()
    delta_res<-c()
    for(j in 1:1){
      flag<-0
      delta<-0
      while(flag==0){
        level<-integrate(f_list[[j]],0+delta,2*pi-delta)$value
        if(level<=1-alpha){
          flag<-1
        }
        delta<-delta+0.00001
      }
      delta_res<-c(delta_res,delta)
      all_deltas <- append(all_deltas,delta_res)
    }
    rejected <- c()
    mu_hat<-Arg(sum(exp(2*1i*gamma)))
    if(abs(mu_hat)<delta_res){
      print(paste("Test 3.4: Reject H_0. Growth occurs along the distal axis, since ", mu_hat, " < ", delta_res, "."))
      rejected <- append(rejected, TRUE)
    }else{
      print(paste("Test 3.4: Growth occurs not along the distal axis, since ", mu_hat, " < ", delta_res, "."))
      rejected <- append(rejected, FALSE)
    }
    return(list("mu_hat"=mu_hat, "delta_res"=delta_res, "rejected"=rejected))
  }
  else{
    g<-list();
    for(i in 1:nr_fingers){
      g<-append(list(gamma[(1+(nr_printsPerFinger*(i-1))):(nr_printsPerFinger*i)]),g)
    }

    # extrinsic mean
    mu_hat<-Arg(sum(exp(2*1i*gamma)))

    mu_neu<-c()
    for(k in 1:100){
      #bootstrap gamma: pro finger 100 samples
      boot_g<-list()
      for(i in 1:length(g)){
        theta <- function(x){x}
        boot_g<-append(boot_g,list(bootstrap(g[[i]],1,theta)$thetastar))
      }

      # f?r jedes der 100 Bootstrapsamples einen mu_hat
      bootstrapsample<-c()
      for(i in 1:nr_fingers){
        bootstrapsample<-c(bootstrapsample,boot_g[[i]])
      }
      mu_neu<-c(mu_neu,(Arg(sum(exp(2*1i*bootstrapsample))))^2)
    }

    bootstrap_var<-var(mu_neu)
    rejected <- c()
    T_compare<-(abs(mu_hat)^2-epsilon^2)/sqrt(bootstrap_var)
    if(T_compare<qnorm(1-alpha)){
      print(paste("We reject the null hypothesis. Growth occurs along the distal axis, since ", T_compare, " < ", qnorm(1-alpha), "."))
      rejected <- append(rejected, TRUE)
    } else{
      print(paste("We do not reject the null hypothesis. There is no distal growth effect, since ", T_compare, " > ", qnorm(1-alpha), "."))
      rejected <- append(rejected, FALSE)
    }
    return(list("T_compare" = T_compare, "qnorm.alpha" = qnorm(1-alpha), "rejected"=rejected))
  }
}

MakeQQPlotEq6 <- function(tau = AnisotropicGrowthEstimation(FirstFingersCentered, OtherFingersCentered)$tau, FirstFingersCentered = FirstFingersCentered, OtherFingersCentered = OtherFingersCentered){
  for(S in 1:56){
    rho<-c()
    n<-dim(FirstFingersCentered[[S]])[2]
    for(k in 1:length(tau)){
      rho[k]<-4*n*log((1+tau[k]/2)/sqrt(1+tau[k]))
    }
    K<-length(rho)
  }

  teststat <- rchisq(56,2); ks.test(teststat,rho)
  par(mfrow=c(1,1), mar = c(5, 6, 4, 2) + 0.1)
  qqplot(rho,teststat,ylab=expression(chi[2]^2), xlab=expression(rho[k]^p),cex.lab=2)
}

