% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Codes.R
\name{AnisotropicGrowthEstimation}
\alias{AnisotropicGrowthEstimation}
\title{Estimate anisotropic growth parameters}
\usage{
AnisotropicGrowthEstimation(FirstFingers, OtherFingers, epsilon = 10^(-10),
  betaStart = pi, lambdaStart = 1.5, gammaStart = 0.5 * pi,
  tauStart = 0.5, iterationsMax = 1000)
}
\arguments{
\item{FirstFingers}{Matrix of centered minutiae coordinates to be compared with or lists thereof.}

\item{OtherFingers}{Matrix of centered minutiae coordinates or lists thereof.}

\item{epsilon}{Accuracy parameter.}

\item{betaStart}{Inital estimate for beta.}

\item{lambdaStart}{Inital estimate for lambda.}

\item{gammaStart}{Inital estimate for gamma.}

\item{tauStart}{Inital estimate for tau.}

\item{iterationsMax}{Number of maximal iterations before abort.}
}
\value{A list with components
\item{lambda}{Isotropic growth rate.}
\item{tau}{Rate of non-isotropic growth.}
\item{beta}{Rotation angles.}
\item{gamma}{Directions of non-istropic growth.}
\item{no_iteration}{Number of iterations of the algorithm.}
}
\description{
Calculate the rotational angle, the rate of isotropic growth, the rate of anisotropic growth and the angle of anisotropic growth for two fingerprints or lists thereof.
}
\examples{

# fingerprint alignment precision
boxplot(Arg(exp(1i*AnisotropicGrowthEstimation(FirstFingersCentered, OtherFingersOrientedCentered)$beta)),xlab=expression(hat(beta)),ylab="",cex.lab=2)

# likelihood ratio test does not apply
tau <-  AnisotropicGrowthEstimation(FirstFingersCentered, OtherFingersCentered)$tau

set.seed(1)
for(S in 1:56){
    rho<-c()
    n<-dim(FirstFingersCentered[[S]])[2]
    for(k in 1:length(tau)){
      rho[k]<-4*n*log((1+tau[k]/2)/sqrt(1+tau[k]))
    }
    K<-length(rho)
  }

teststat <- rchisq(56,2); ks.test(teststat,rho)
par(mfrow=c(1,1), mar = c(5, 6, 4, 2) + 0.1)
qqplot(rho,teststat,ylab=expression(chi[2]^2), xlab=expression(rho[k]^p),cex.lab=2)

# in case of no growth, tau is overestimated and lambda is underestimated

A <- AnisotropicGrowthEstimation(FirstFingersCentered, OtherFingersCentered)
hat.tau <- A$tau; hat.lambda <- A$lambda

par(mfrow=c(1,2))
boxplot(hat.tau,xlab=expression(hat(tau)),cex.lab=2,cex.axis=2)
boxplot(hat.lambda, xlab=expression(hat(lambda)),cex.lab=2,cex.axis=2)

# simulating and testing for moderate growth
     sigma2 <- 0.1 
      sigma3 <- 0.2
      set.seed(1)
      taus <- 0.0 + rtruncnorm(56, a=0, b=sigma2*5, mean = 0, sd = sigma2)
      lambdas <- 1.0 + rtruncnorm(56, a=0, b=sigma3*5, mean = 0, sd = sigma3)
OtherFingersGrown <- SimulateAnisotropicGrowth(OtherFingersOrientedCentered,taus=taus,lambdas=lambdas)    
A <- AnisotropicGrowthEstimation(FirstFingersCentered, OtherFingersGrown)
hat.tau <- A$tau; hat.lambda <- A$lambda; hat.gamma <- A$gamma
 
par(mfrow=c(2,2),mgp=c(0,2,0))
boxplot(taus, hat.tau,names=c(expression(tau),expression(hat(tau))),cex.lab=2,cex.axis=2)
boxplot(lambdas, hat.lambda, names=c(expression(lambda),expression(hat(lambda))),cex.lab=2,cex.axis=2)
RoseDiagram(2*A$gamma, cex=2, main=NULL)

median(lambdas)
median(hat.lambda) 
median(taus) 
median(hat.tau) 

TestForDistalGrowth(hat.gamma) 
TestForDistalGrowth(hat.gamma,epsilon=0.72) 
TestForDistalGrowth(hat.gamma,epsilon=0.73) 

TestForDistalGrowth(hat.gamma,parametric=FALSE) 

TestForGamma(gamma=hat.gamma, alpha=0.01)
      generatedFingers = SimulatePoissonFingers(xlimits=c(-95,95), ylimits=c(-165,165), sigma=7,poissonExpectedValue = 22)
      sim_taus = AnisotropicGrowthEstimation(generatedFingers$FirstFingers, generatedFingers$OtherFingers)$tau
TestForTau(tau = taus, reference_taus = sim_taus)

# simulating and testing for strong growth
      sigma2 <- 0.8 
      sigma3 <- 0.8
      set.seed(1)
      taus <- 0.0 + rtruncnorm(56, a=0, b=sigma2*5, mean = 0, sd = sigma2)
      lambdas <- 1.0 + rtruncnorm(56, a=0, b=sigma3*5, mean = 0, sd = sigma3)
OtherFingersGrown <- SimulateAnisotropicGrowth(OtherFingersOrientedCentered,taus=taus,lambdas=lambdas)    
A <- AnisotropicGrowthEstimation(FirstFingersCentered, OtherFingersGrown)
hat.tau <- A$tau; hat.lambda <- A$lambda; hat.gamma <- A$gamma

par(mfrow=c(2,2),mgp=c(0,2,0))
boxplot(taus, hat.tau,names=c(expression(tau),expression(hat(tau))),cex.lab=2,cex.axis=2)
boxplot(lambdas, hat.lambda, names=c(expression(lambda),expression(hat(lambda))),cex.lab=2,cex.axis=2)
RoseDiagram(2*A$gamma, main=NULL)

median(lambdas) 
median(hat.lambda) 
median(taus) 
median(hat.tau) 

TestForDistalGrowth(hat.gamma)  
TestForDistalGrowth(hat.gamma,epsilon=0.33) 
TestForDistalGrowth(hat.gamma,epsilon=0.32) 

TestForDistalGrowth(hat.gamma,parametric=FALSE) 

TestForGamma(gamma=hat.gamma, alpha=0.01)
      generatedFingers = SimulatePoissonFingers(xlimits=c(-95,95), ylimits=c(-165,165), sigma=7,poissonExpectedValue = 22)
      sim_taus = AnisotropicGrowthEstimation(generatedFingers$FirstFingers, generatedFingers$OtherFingers)$tau
TestForTau(tau = taus, reference_taus = sim_taus)



}
